#include "util.h"
#include <QApplication>
#include <QComboBox>
#include <QLabel>
#include <QMainWindow>
#include <QMenu>
#include <QMenuBar>
#include <QPlainTextEdit>
#include <QVBoxLayout>

#include <QMouseEvent>

#include <QDebug>

#include <algorithm>
#include <iterator>

class MouseLabel : public QLabel
{
       Q_OBJECT
public:
    MouseLabel() : QLabel("Click")
    {
        setObjectName("MouseLabel");
        setFrameShape(QFrame::Box);
        setAlignment(Qt::AlignCenter);
    }

protected:
     void mousePressEvent(QMouseEvent *event) override
     {
         auto p = event->pos();
         auto globalP = mapToGlobal(p);
         auto mappedBack = mapFromGlobal(globalP);
         qDebug() << "MouseLabel" << p << "globalP=" << globalP << "mappedBack=" << mappedBack
             << "\n  QCursor=" << QCursor::pos() << "nativeCursor=" << nativeCursorPosition();
     }
};

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow();

public slots:
    void updateTitle();
    void updateText();
    void aboutToShow();
    void actionTriggered();
    void dumpGeometry();
    void screenChanged();

protected:
    void resizeEvent(QResizeEvent *event) override;
    void moveEvent(QMoveEvent *event) override;

private:
    QPlainTextEdit *m_pe;
};

MainWindow::MainWindow()
{
    setObjectName("Main");

    auto mb = menuBar();
    mb->setObjectName("MenuBar");
    for (int i = 0; i < 10; ++i) {
        QString text = "New Menu";
        if (i < 10)
            text += '&';
        text += QString::number(i);
        auto m = mb->addMenu(text);
        m->setObjectName(text);

        if (!i) {
            auto da = m->addAction("Dump", this, &MainWindow::dumpGeometry);
            da->setShortcut(Qt::CTRL + Qt::Key_D);
        }

        m->addAction("&Test action", this, &MainWindow::actionTriggered);
        connect(m, &QMenu::aboutToShow, this, &MainWindow::aboutToShow);

        if (!i) {
            auto qa = m->addAction("Quit", this, &QWidget::close);
            qa->setShortcut(Qt::CTRL + Qt::Key_Q);
        }

    }

    auto cw = new QWidget;
    cw->setObjectName("CentralWidget");
    auto cl = new QVBoxLayout(cw);
    cl->addWidget(new MouseLabel);

    auto hl = new QHBoxLayout;
    hl->addStretch();
    auto ttL = new QLabel("toolTip");
    ttL->setToolTip("ToolTip!");
    hl->addWidget(ttL);
    auto cb = new QComboBox;
    hl->addWidget(cb);
    cb->addItems(QStringList{"item1","item2","item3","item4"});
    cl->addLayout(hl);

    m_pe = new QPlainTextEdit();
    m_pe->setObjectName("PlainTextEdit");
    cl->addWidget(m_pe);
    setCentralWidget(cw);
}

void MainWindow::aboutToShow()
{
    auto menu = qobject_cast<QMenu *>(sender());
    if (!menu)
        return;
    QDebug d = qDebug().noquote();
    d << "aboutToShow()" << menu << menu->geometry()
      << "dpr=" << menu->devicePixelRatioF();
    if (auto screen = screenOf(menu))
        d << screen->name() << "dpr=" << screen->devicePixelRatio();
    else
        d << "screen=nullptr";
}

void MainWindow::actionTriggered()
{
    if (auto a = qobject_cast<QAction *>(sender())) {
        if (auto w = a->associatedWidgets().value(0))
            qDebug().noquote() << "actionTriggered()" << w << w->geometry() << screenName(w);
    }
}

void MainWindow::screenChanged()
{
    qDebug().noquote() << "Screen changed: " << screenName(this);
    updateTitle();
    updateText();
}

void MainWindow::updateTitle()
{
    QString t = screenName(this) + QLatin1Char(' ') + QLatin1String(QT_VERSION_STR)
        + QLatin1Char(' ') + QGuiApplication::platformName();
    setWindowTitle(t);
}

static QString widgetDescr(const QWidget *w)
{
    QString s;
    QTextStream str(&s);
    str << w->geometry() << " \"" << qWindowOf(w)->screen()->name() << "\", dpr=" << w->devicePixelRatioF();
    return s;
}

void MainWindow::updateText()
{
      m_pe->setPlainText(widgetDescr(this) + "\n\n" + formatScreens(this));
}

void MainWindow::dumpGeometry()
{
    updateText();
    qDebug().noquote() << __FUNCTION__ << widgetDescr(this);
}

void MainWindow::resizeEvent(QResizeEvent *event)
{
    updateText();
    QMainWindow::resizeEvent(event);
}

void MainWindow::moveEvent(QMoveEvent *event)
{
    updateText();
    QMainWindow::moveEvent(event);
}

int main(int argc, char **argv)
{
    QStringList args;
    std::copy(argv + 1, argv + argc, std::back_inserter(args));

    if (args.contains("-n"))
        QCoreApplication::setAttribute(Qt::AA_DisableHighDpiScaling);
    else
        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);

    QApplication a(argc, argv);

    EventFilter::logInputEvents = args.contains("-i");
    if (!args.contains("-e"))
        a.installEventFilter(new EventFilter(&a));

    QString t =  QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') + QGuiApplication::platformName();
    qDebug().noquote().nospace() << t << ' ' << args << '\n' << formatScreens();

    MainWindow mw;
    if (auto s = QGuiApplication::screenAt(QCursor::pos())) {
        auto g = s->geometry();
        mw.resize(g.width() * 3 / 4, 100);
    }
    mw.show();
    QObject::connect(mw.windowHandle(), &QWindow::screenChanged, &mw, &MainWindow::screenChanged);
    mw.updateTitle();
    mw.updateText();
    return a.exec();
}

#include "main.moc"
