#include "util.h"

#include <QDebug>
#include <QGuiApplication>
#include <QMouseEvent>

#ifdef Q_OS_WIN
#  include <qt_windows.h>
#endif

const QWindow *qWindowOf(const QWidget *w)
{
    auto handle = w->windowHandle();
    return handle ? handle : w->window()->windowHandle();
}

const QScreen *screenOf(const QObject *o)
{
    if (o->isWindowType())
        return static_cast<const QWindow *>(o)->screen();
    if (o->isWidgetType()) {
        if (auto handle = qWindowOf(static_cast<const QWidget *>(o)))
            return handle->screen();
    }
    return nullptr;
}

QString screenName(const QWidget *w)
{
     auto wh = qWindowOf(w);
     return wh ? wh->screen()->name() : QString();
}

QTextStream &operator<<(QTextStream &s, const QSize &sz)
{
    s << sz.width() << 'x' << sz.height();
    return s;
}

QTextStream &operator<<(QTextStream &s, const QRect &r)
{
    s << r.size() << forcesign << r.left() << r.top() << noforcesign;
    return s;
}

QPoint nativeCursorPosition()
{
    QPoint qp;
#ifdef Q_OS_WIN
    POINT wp;
    GetCursorPos(&wp);
    qp.setX(wp.x);
    qp.setY(wp.y);
#endif
    return qp;
}

QString formatScreens(const QWidget *w)
{
    const QScreen *widgetScreen = nullptr;
    if (w) {
          if (auto qw = qWindowOf(w))
              widgetScreen = qw->screen();
    }

    QString s;
    QTextStream str(&s);
    for (auto screen : QGuiApplication::screens()) {
        str << "  " << (screen == widgetScreen ? '*' : ' ')
            << " \"" << screen->name() << "\" " << screen->geometry()
            << " dpr=" << screen->devicePixelRatio() << '\n';
    }
    return s;
}

QDebug operator<<(QDebug  d, const formatObject &f)
{
    QDebugStateSaver s(d);
    d.noquote();
    d.nospace();
    auto on = f.m_o->objectName();
    if (!on.isEmpty())
        d << '"' << on << "\"/";
    d << f.m_o->metaObject()->className();
    if (on.isEmpty())
        d << '(' << static_cast<const void*>(f.m_o) << ')';
    return d;
}

bool EventFilter::logInputEvents = false;

template <class Event>
void dumpInputEvent(QObject *watched, QEvent *event, const char *desc)
{
    auto ie = static_cast<const Event *>(event);
    qDebug() << desc << formatObject(watched) << ie->pos() << "global=" << ie->globalPos();
}

bool EventFilter::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type()) {
    case QEvent::ScreenChangeInternal:
        qDebug() << formatObject(watched) << event << screenOf(watched);
        break;
    case QEvent::NonClientAreaMouseButtonPress:
    case QEvent::NonClientAreaMouseButtonDblClick:
    case QEvent::MouseButtonPress:
    case QEvent::MouseButtonDblClick:
        if (logInputEvents)
            dumpInputEvent<QMouseEvent>(watched, event, "Mouse");
        break;
    case QEvent::Enter:
        if (logInputEvents)
            dumpInputEvent<QEnterEvent>(watched, event, "Enter");
        break;
    case QEvent::ContextMenu:
        if (logInputEvents)
            dumpInputEvent<QContextMenuEvent>(watched, event, "CtxMenu");
        break;
    default:
        break;
    }
    return false;
}

